#include "States/GameState.h"
#include "States/SelectState.h"
#include "Events/IGameEvent.h"
//#include "Events/LostGame.hpp" ///@todo убрать
//#include "Events/DeleteStaticEntity.hpp" ///@todo убрать

GameState::GameState(IStateManager& state_manager, const sf::VideoMode& video_mode, const std::string& window_title, float width, float height):
        IWindowKeeper(video_mode, window_title), IState(state_manager), m_maze_width(width), m_maze_height(height) {}

void GameState::set_maze(Maze&& maze) {
    m_maze = std::move(maze);
}

void GameState::set_context(GameContext&& context) {
    m_context_manager.reset(std::move(context));
}

bool GameState::do_step() {
    while (m_window.isOpen()) {
        event_handling();
        update();
        render();
        return true;
    }
}

void GameState::process_key_pressed(sf::Keyboard::Key key) {
    if (key == config::KEY_LEFT) {
        m_context_manager.save_current_context();
        m_context_manager.get_current_context().pacman.move(Room::LEFT);
    }
    else if (key == config::KEY_RIGHT) {
        m_context_manager.save_current_context();
        m_context_manager.get_current_context().pacman.move(Room::RIGHT);
    }
    else if (key == config::KEY_UP) {
        m_context_manager.save_current_context();
        m_context_manager.get_current_context().pacman.move(Room::UP);
    }
    else if (key == config::KEY_DOWN) {
        m_context_manager.save_current_context();
        m_context_manager.get_current_context().pacman.move(Room::DOWN);
    }
}

void GameState::process_entities_interactions(IVisitor* visitor) {
    std::unique_ptr<IGameEvent> static_event;
    std::unique_ptr<IGameEvent> dynamic_event;
    for (auto& obj : m_context_manager.get_current_context().static_objects) {
        static_event = std::move(obj->accept(visitor));
        if (static_event)
            break;
    }
    for (auto& obj : m_context_manager.get_current_context().dynamic_objects) {
        dynamic_event = std::move(obj->accept(visitor));
        if (dynamic_event)
            break;
    }
    if (static_event)
        static_event->handle(m_context_manager.get_current_context());
    if (dynamic_event)
        dynamic_event->handle(m_context_manager.get_current_context());
}



void GameState::event_handling() {
    sf::Event event;
    while (m_window.pollEvent(event)) {
        if (event.type == sf::Event::Closed) {
            m_state_manager.set_next_state(std::make_unique<SelectState>(m_state_manager,
            config::SELECT_LEVEL_VIDEO_MODE, config::SELECT_LEVEL_TITLE));
        }
        if (event.type == sf::Event::Resized) {
            sf::View view = m_window.getView();
            view.setSize(event.size.width, event.size.height);
            view.setCenter(m_maze_width / 2.f, m_maze_height / 2.f);
            m_window.setView(view);
        }
        if (event.type == sf::Event::KeyPressed && event.key.control && event.key.code == sf::Keyboard::Z) 
            m_context_manager.restore_previous_context();
        if (event.type == sf::Event::KeyPressed && m_context_manager.get_current_context().state == GameContext::INGAME) {
            process_key_pressed(event.key.code);
            break;
        }
    }
}

void GameState::update() {
    if (m_context_manager.get_current_context().state == GameContext::LOST ||
        m_context_manager.get_current_context().state == GameContext::WIN) {
        return;
    }
    for (auto& obj : m_context_manager.get_current_context().dynamic_objects) {
        obj->action();
    }
    process_entities_interactions(&m_context_manager.get_current_context().pacman);
    if (!m_context_manager.get_current_context().static_objects.size())
        m_context_manager.get_current_context().state = GameContext::WIN;
}

void GameState::render() {
    if (m_context_manager.get_current_context().state == GameContext::INGAME) {
        m_window.clear(config::GAME_COLOR_BACKGROUND_INGAME);
    }
    else if (m_context_manager.get_current_context().state == GameContext::WIN) {
        m_window.clear(config::GAME_COLOR_BACKGROUND_WIN);
    }
    else if (m_context_manager.get_current_context().state == GameContext::LOST) {
        m_window.clear(config::GAME_COLOR_BACKGROUND_LOST);
    }
    m_maze.draw_into(m_window);
    m_context_manager.get_current_context().pacman.draw_into(m_window);
    for (auto& obj : m_context_manager.get_current_context().static_objects) {
        obj->draw_into(m_window);
    }
    for (auto& obj : m_context_manager.get_current_context().dynamic_objects) {
        obj->draw_into(m_window);
    }
    m_window.display();
}